
from __future__ import absolute_import, print_function, division

import unittest

from foalorm.orm.core import *
from foalorm.orm.tests.testutils import *
from foalorm.orm.tests import teardown_database, setup_database

db = Database()


class Group(db.Entity):
    id = PrimaryKey(int)
    number = Required(str, unique=True)
    students = Set("Student")


class Student(db.Entity):
    id = PrimaryKey(int)
    name = Required(str)
    group = Required("Group")


class TestCRUD(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        setup_database(db)
        with db_session:
            g1 = Group(id=1, number='g111')
            g2 = Group(id=2, number='g222')
            s1 = Student(id=1, name='S1', group=g1)
            s2 = Student(id=2, name='S2', group=g1)
            s3 = Student(id=3, name='S3', group=g2)

    @classmethod
    def tearDownClass(cls):
        teardown_database(db)

    def setUp(self):
        rollback()
        db_session.__enter__()

    def tearDown(self):
        rollback()
        db_session.__exit__()

    def test_unique_load(self):
        s1 = Student[1]
        g1 = s1.group
        g1.number = 'g123'
        self.assertEqual(g1.number, 'g123')



from __future__ import absolute_import, print_function, division

import threading
import unittest

from foalorm.orm.core import *


db = Database('sqlite', ':sharedmemory:')


class Person(db.Entity):
    name = Required(str)

db.generate_mapping(create_tables=True)

with db_session:
    Person(name='John')
    Person(name='Mike')


class TestThread(threading.Thread):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, *kwargs)
        self.result = []
    def run(self):
        with db_session:
            persons = Person.select().fetch()
            self.result.extend(p.name for p in persons)


class TestFlush(unittest.TestCase):
    def test1(self):
        thread1 = TestThread()
        thread1.start()
        thread1.join()
        self.assertEqual(set(thread1.result), {'John', 'Mike'})



from flask import Flask
from flask_login import LoginManager
from foalorm.flask import FoalORM
from .config import config
from .models import db

app = Flask(__name__)
app.config.update(config)

FoalORM(app)
login_manager = LoginManager(app)
login_manager.login_view = 'login'

@login_manager.user_loader
def load_user(user_id):
    return db.User.get(id=user_id)



# coding: utf-8

from __future__ import absolute_import, print_function, division

from binascii import unhexlify
import unittest

from foalorm.orm.core import *
from foalorm.orm.tests.testutils import *
from foalorm.orm.tests import only_for

db = Database('sqlite', ':memory:')


class Person(db.Entity):
    name = Required(str)
    age = Optional(int)
    image = Optional(buffer)


db.generate_mapping(create_tables=True)

with db_session:
    p1 = Person(name='John', age=20, image=unhexlify('abcdef'))
    p2 = Person(name=u'Иван')  # u'\u0418\u0432\u0430\u043d'


@only_for('sqlite')
class TestUnicode(unittest.TestCase):
    @db_session
    def test1(self):
        names = select(p.name for p in Person).order_by(lambda: p.id)[:]
        self.assertEqual(names, ['John', u'Иван'])

    @db_session
    def test2(self):
        names = select(p.name.upper() for p in Person).order_by(lambda: p.id)[:]
        self.assertEqual(names, ['JOHN', u'ИВАН'])  # u'\u0418\u0412\u0410\u041d'

    @db_session
    def test3(self):
        names = select(p.name.lower() for p in Person).order_by(lambda: p.id)[:]
        self.assertEqual(names, ['john', u'иван'])  # u'\u0438\u0432\u0430\u043d'

    @db_session
    def test4(self):
        ages = db.select('select py_upper(age) from person')
        self.assertEqual(ages, ['20', None])

    @db_session
    def test5(self):
        ages = db.select('select py_lower(age) from person')
        self.assertEqual(ages, ['20', None])

    @db_session
    def test6(self):
        ages = db.select('select py_upper(image) from person')
        self.assertEqual(ages, [u'ABCDEF', None])

    @db_session
    def test7(self):
        ages = db.select('select py_lower(image) from person')
        self.assertEqual(ages, [u'abcdef', None])


if __name__ == '__main__':
    unittest.main()



from __future__ import absolute_import, print_function, division

import unittest
from foalorm.orm.sqlbuilding import SQLBuilder
from foalorm.orm.dbapiprovider import DBAPIProvider
from foalorm.orm.tests.testutils import TestPool


class TestFormatStyles(unittest.TestCase):
    def setUp(self):
        self.key1 = 'KEY1'
        self.key2 = 'KEY2'
        self.provider = DBAPIProvider(_database=None, foalorm_pool_mockup=TestPool(None))
        self.ast = [ 'SELECT', [ 'ALL', ['COLUMN', None, 'A']], [ 'FROM', [None, 'TABLE', 'T1']],
                     [ 'WHERE', [ 'EQ', ['COLUMN', None, 'B'], [ 'PARAM', self.key1 ] ],
                              [ 'EQ', ['COLUMN', None, 'C'], [ 'PARAM', self.key2 ] ],
                              [ 'EQ', ['COLUMN', None, 'D'], [ 'PARAM', self.key2 ] ],
                              [ 'EQ', ['COLUMN', None, 'E'], [ 'PARAM', self.key1 ] ]
                     ]
                   ]
    def test_qmark(self):
        self.provider.paramstyle = 'qmark'
        b = SQLBuilder(self.provider, self.ast)
        self.assertEqual(b.sql, 'SELECT "A"\n'
                                'FROM "T1"\n'
                                'WHERE "B" = ?\n  AND "C" = ?\n  AND "D" = ?\n  AND "E" = ?')
        self.assertEqual(b.layout, [self.key1, self.key2, self.key2, self.key1])
    def test_numeric(self):
        self.provider.paramstyle = 'numeric'
        b = SQLBuilder(self.provider, self.ast)
        self.assertEqual(b.sql, 'SELECT "A"\n'
                                'FROM "T1"\n'
                                'WHERE "B" = :1\n  AND "C" = :2\n  AND "D" = :2\n  AND "E" = :1')
        self.assertEqual(b.layout, [self.key1, self.key2, self.key2, self.key1])
    def test_named(self):
        self.provider.paramstyle = 'named'
        b = SQLBuilder(self.provider, self.ast)
        self.assertEqual(b.sql, 'SELECT "A"\n'
                                'FROM "T1"\n'
                                'WHERE "B" = :p1\n  AND "C" = :p2\n  AND "D" = :p2\n  AND "E" = :p1')
        self.assertEqual(b.layout, [self.key1, self.key2, self.key2, self.key1])
    def test_format(self):
        self.provider.paramstyle = 'format'
        b = SQLBuilder(self.provider, self.ast)
        self.assertEqual(b.sql, 'SELECT "A"\n'
                                'FROM "T1"\n'
                                'WHERE "B" = %s\n  AND "C" = %s\n  AND "D" = %s\n  AND "E" = %s')
        self.assertEqual(b.layout, [self.key1, self.key2, self.key2, self.key1])
    def test_pyformat(self):
        self.provider.paramstyle = 'pyformat'
        b = SQLBuilder(self.provider, self.ast)
        self.assertEqual(b.sql, 'SELECT "A"\n'
                                'FROM "T1"\n'
                                'WHERE "B" = %(p1)s\n  AND "C" = %(p2)s\n  AND "D" = %(p2)s\n  AND "E" = %(p1)s')
        self.assertEqual(b.layout, [self.key1, self.key2, self.key2, self.key1])


if __name__ == "__main__":
    unittest.main()



import unittest
from decimal import Decimal
from datetime import date

from foalorm.orm import *
from foalorm.orm.serialization import to_dict
from foalorm.orm.tests.testutils import *
from foalorm.orm.tests import setup_database, teardown_database


class TestObjectToDict(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        db = cls.db = Database()

        class Student(db.Entity):
            name = Required(str)
            scholarship = Optional(int)
            gpa = Optional(Decimal, 3, 1)
            dob = Optional(date)
            group = Optional('Group')
            courses = Set('Course')
            biography = Optional(LongUnicode)

        class Group(db.Entity):
            number = PrimaryKey(int)
            students = Set(Student)

        class Course(db.Entity):
            name = Required(str, unique=True)
            students = Set(Student)

        setup_database(db)

        with db_session:
            g1 = Group(number=1)
            g2 = Group(number=2)
            c1 = Course(id=1, name='Math')
            c2 = Course(id=2, name='Physics')
            c3 = Course(id=3, name='Computer Science')
            Student(id=1, name='S1', group=g1, gpa=3.1, courses=[c1, c2], biography='some text')
            Student(id=2, name='S2', group=g1, gpa=3.2, scholarship=100, dob=date(2000, 1, 1))
            Student(id=3, name='S3', group=g1, gpa=3.3, scholarship=200, dob=date(2001, 1, 2), courses=[c2, c3])
            Student(id=4, name='S4')

    @classmethod
    def tearDownClass(cls):
        teardown_database(cls.db)

    def setUp(self):
        rollback()
        db_session.__enter__()

    def tearDown(self):
        rollback()
        db_session.__exit__()

    def test1(self):
        s1 = self.db.Student[1]
        d = s1.to_dict()
        self.assertEqual(d, dict(id=1, name='S1', scholarship=None, gpa=Decimal('3.1'), dob=None,
                                 group=1))

    def test2(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(related_objects=True)
        self.assertEqual(d, dict(id=1, name='S1', scholarship=None, gpa=Decimal('3.1'), dob=None,
                                 group=self.db.Group[1]))

    def test3(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(with_collections=True)
        self.assertEqual(d, dict(id=1, name='S1', scholarship=None, gpa=Decimal('3.1'), dob=None,
                                 group=1, courses=[1, 2]))

    def test4(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(with_collections=True, related_objects=True)
        self.assertEqual(d, dict(id=1, name='S1', scholarship=None, gpa=Decimal('3.1'), dob=None,
                                 group=self.db.Group[1], courses=[self.db.Course[1], self.db.Course[2]]))

    def test5(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(with_lazy=True)
        self.assertEqual(d, dict(id=1, name='S1', scholarship=None, gpa=Decimal('3.1'), dob=None,
                                 group=1, biography='some text'))

    def test6(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(only=['id', 'name', 'group'])
        self.assertEqual(d, dict(id=1, name='S1', group=1))

    def test7(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(['id', 'name', 'group'])
        self.assertEqual(d, dict(id=1, name='S1', group=1))

    def test8(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(only='id, name, group')
        self.assertEqual(d, dict(id=1, name='S1', group=1))

    def test9(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(only='id name group')
        self.assertEqual(d, dict(id=1, name='S1', group=1))

    def test10(self):
        s1 = self.db.Student[1]
        d = s1.to_dict('id name group')
        self.assertEqual(d, dict(id=1, name='S1', group=1))

    @raises_exception(AttributeError, 'Entity Student does not have attribute x')
    def test11(self):
        s1 = self.db.Student[1]
        d = s1.to_dict('id name x group')
        self.assertEqual(d, dict(id=1, name='S1', group=1))

    def test12(self):
        s1 = self.db.Student[1]
        d = s1.to_dict('id name group', related_objects=True)
        self.assertEqual(d, dict(id=1, name='S1', group=self.db.Group[1]))

    def test13(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(exclude=['dob', 'gpa', 'scholarship'])
        self.assertEqual(d, dict(id=1, name='S1', group=1))

    def test14(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(exclude='dob, gpa, scholarship')
        self.assertEqual(d, dict(id=1, name='S1', group=1))

    def test15(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(exclude='dob gpa scholarship')
        self.assertEqual(d, dict(id=1, name='S1', group=1))

    @raises_exception(AttributeError, 'Entity Student does not have attribute x')
    def test16(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(exclude='dob gpa x scholarship')
        self.assertEqual(d, dict(id=1, name='S1', group=1))

    def test17(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(exclude='dob gpa scholarship', related_objects=True)
        self.assertEqual(d, dict(id=1, name='S1', group=self.db.Group[1]))

    def test18(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(exclude='dob gpa scholarship', with_lazy=True)
        self.assertEqual(d, dict(id=1, name='S1', group=1, biography='some text'))

    def test19(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(exclude='dob gpa scholarship biography', with_lazy=True)
        self.assertEqual(d, dict(id=1, name='S1', group=1))

    def test20(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(exclude='dob gpa scholarship', with_collections=True)
        self.assertEqual(d, dict(id=1, name='S1', group=1, courses=[1, 2]))

    def test21(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(exclude='dob gpa scholarship courses', with_collections=True)
        self.assertEqual(d, dict(id=1, name='S1', group=1))

    def test22(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(only='id name group', exclude='dob group')
        self.assertEqual(d, dict(id=1, name='S1'))

    def test23(self):
        s1 = self.db.Student[1]
        d = s1.to_dict(only='id name group', exclude='dob group', with_collections=True, with_lazy=True)
        self.assertEqual(d, dict(id=1, name='S1'))

    def test24(self):
        c = self.db.Course(id=4, name='New Course')
        d = c.to_dict()  # should do flush and get c.id from the database
        self.assertEqual(d, dict(id=4, name='New Course'))


class TestSerializationToDict(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        db = cls.db = Database()

        class Student(db.Entity):
            name = Required(str)
            scholarship = Optional(int)
            gpa = Optional(Decimal, 3, 1)
            dob = Optional(date)
            group = Optional('Group')
            courses = Set('Course')
            biography = Optional(LongUnicode)

        class Group(db.Entity):
            number = PrimaryKey(int)
            students = Set(Student)

        class Course(db.Entity):
            name = Required(str, unique=True)
            students = Set(Student)

        setup_database(db)

        with db_session:
            g1 = Group(number=1)
            g2 = Group(number=2)
            c1 = Course(name='Math')
            c2 = Course(name='Physics')
            c3 = Course(name='Computer Science')
            Student(id=1, name='S1', group=g1, gpa=3.1, courses=[c1, c2], biography='some text')
            Student(id=2, name='S2', group=g1, gpa=3.2, scholarship=100, dob=date(2000, 1, 1))
            Student(id=3, name='S3', group=g1, gpa=3.3, scholarship=200, dob=date(2001, 1, 2), courses=[c2, c3])
            Student(id=4, name='S4')

    @classmethod
    def tearDownClass(cls):
        teardown_database(cls.db)

    def setUp(self):
        rollback()
        db_session.__enter__()

    def tearDown(self):
        rollback()
        db_session.__exit__()

    def test1(self):
        s4 = self.db.Student[4]
        self.assertEqual(s4.group, None)
        d = to_dict(s4)
        self.assertEqual(d, dict(Student={
            4 : dict(id=4, name='S4', group=None, dob=None, gpa=None, scholarship=None, courses=[])
            }))

if __name__ == '__main__':
    unittest.main()



import sys, unittest

from foalorm.orm import *
from foalorm.orm.tests.testutils import *
from foalorm.orm.tests import setup_database, teardown_database


class TestVolatile1(unittest.TestCase):
    def setUp(self):
        db = self.db = Database()

        class Item(self.db.Entity):
            name = Required(str)
            index = Required(int, volatile=True)

        setup_database(db)

        with db_session:
            Item(id=1, name='A', index=1)
            Item(id=2, name='B', index=2)
            Item(id=3, name='C', index=3)

    def tearDown(self):
        teardown_database(self.db)

    @db_session
    def test_1(self):
        db = self.db
        Item = db.Item
        db.execute('update "item" set "index" = "index" + 1')
        items = Item.select(lambda item: item.index > 0).order_by(Item.id)[:]
        a, b, c = items
        self.assertEqual(a.index, 2)
        self.assertEqual(b.index, 3)
        self.assertEqual(c.index, 4)
        c.index = 1
        items = Item.select()[:]  # force re-read from the database
        self.assertEqual(c.index, 1)
        self.assertEqual(a.index, 2)
        self.assertEqual(b.index, 3)


    @db_session
    def test_2(self):
        Item = self.db.Item
        item = Item[1]
        item.name = 'X'
        item.flush()
        self.assertEqual(item.index, 1)


class TestVolatile2(unittest.TestCase):
    def setUp(self):
        db = self.db = Database()

        class Group(db.Entity):
            number = PrimaryKey(int)
            students = Set('Student', volatile=True)

        class Student(db.Entity):
            id = PrimaryKey(int)
            name = Required(str)
            group = Required('Group')
            courses = Set('Course')

        class Course(db.Entity):
            id = PrimaryKey(int)
            name = Required(str)
            students = Set('Student', volatile=True)

        setup_database(db)

        with db_session:
            g1 = Group(number=123)
            s1 = Student(id=1, name='A', group=g1)
            s2 = Student(id=2, name='B', group=g1)
            c1 = Course(id=1, name='C1', students=[s1, s2])
            c2 = Course(id=2, name='C1', students=[s1])

        self.Group = Group
        self.Student = Student
        self.Course = Course

    def tearDown(self):
        teardown_database(self.db)

    def test_1(self):
        self.assertTrue(self.Group.students.is_volatile)
        self.assertTrue(self.Student.group.is_volatile)
        self.assertTrue(self.Student.courses.is_volatile)
        self.assertTrue(self.Course.students.is_volatile)

    def test_2(self):
        with db_session:
            g1 = self.Group[123]
            students = set(s.id for s in g1.students)
            self.assertEqual(students, {1, 2})
            self.db.execute('''insert into student values(3, 'C', 123)''')
            g1.students.load()
            students = set(s.id for s in g1.students)
            self.assertEqual(students, {1, 2, 3})

    def test_3(self):
        with db_session:
            g1 = self.Group[123]
            students = set(s.id for s in g1.students)
            self.assertEqual(students, {1, 2})
            self.db.execute("insert into student values(3, 'C', 123)")
            s3 = self.Student[3]
            students = set(s.id for s in g1.students)
            self.assertEqual(students, {1, 2, 3})

    def test_4(self):
        with db_session:
            c1 = self.Course[1]
            students = set(s.id for s in c1.students)
            self.assertEqual(students, {1, 2})
            self.db.execute("insert into student values(3, 'C', 123)")
            attr = self.Student.courses
            self.db.execute("insert into %s values(1, 3)" % attr.table)
            c1.students.load()
            students = set(s.id for s in c1.students)
            self.assertEqual(students, {1, 2, 3})


if __name__ == '__main__':
    unittest.main()



from __future__ import print_function

from setuptools import setup
import sys

import unittest

def test_suite():
    test_loader = unittest.TestLoader()
    test_suite = test_loader.discover('foalorm.orm.tests', pattern='test_*.py')
    return test_suite

name = "foalorm"
version = __import__('foalorm').__version__
description = "Foal Object-Relational Mapper"
long_description = """
About
=========
FoalORM is an advanced object-relational mapper forked from PonyORM.

Installation
=================
::

    pip install foalorm

"""

classifiers = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: Apache Software License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
    'Programming Language :: Python :: 3.11',
    'Programming Language :: Python :: Implementation :: PyPy',
    'Topic :: Software Development :: Libraries',
    'Topic :: Database'
]

author_email = "me@frodo821.me"
author = f'frodo821 <{author_email}>'
project_urls = {
    "Source": "https://github.com/frodo821/foalorm",
}
license = "MIT License"

packages = [
    "foalorm",
    "foalorm.flask",
    "foalorm.flask.example",
    "foalorm.orm",
    "foalorm.orm.dbproviders",
    "foalorm.orm.examples",
    "foalorm.orm.integration",
    "foalorm.orm.tests",
    "foalorm.thirdparty",
    "foalorm.utils"
]

package_data = {
    'foalorm.flask.example': ['templates/*.html'],
    'foalorm.orm.tests': ['queries.txt']
}

download_url = "http://pypi.python.org/pypi/pony/"

if __name__ == "__main__":
    pv = sys.version_info[:2]
    if pv not in ((3, 6), (3, 7), (3, 8), (3, 9), (3, 10), (3, 11)):
        s = "Sorry, but %s %s requires Python of one of the following versions: 3.6-3.11." \
            " You have version %s"
        print(s % (name, version, sys.version.split(' ', 1)[0]))
        sys.exit(1)

    setup(
        name=name,
        version=version,
        description=description,
        long_description=long_description,
        classifiers=classifiers,
        author=author,
        author_email=author_email,
        project_urls=project_urls,
        license=license,
        packages=packages,
        package_data=package_data,
        download_url=download_url,
        test_suite='setup.test_suite'
    )



import pickle, unittest

from foalorm.orm import *
from foalorm.orm.tests.testutils import raises_exception, raises_if
from foalorm.orm.ormtypes import Json, TrackedValue, TrackedList, TrackedDict
from foalorm.orm.tests import setup_database, teardown_database

db = Database()


class Product(db.Entity):
    name = Required(str)
    info = Optional(Json)
    tags = Optional(Json)


class TestJson(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        setup_database(db)

    def setUp(self):
        with db_session:
            Product.select().delete(bulk=True)
            flush()
            Product(
                name='Apple iPad Air 2',
                info={
                    'name': 'Apple iPad Air 2',
                    'display': {
                     'size': 9.7,
                     'resolution': [2048, 1536],
                     'matrix-type': 'IPS',
                     'multi-touch': True
                    },
                    'os': {
                     'type': 'iOS',
                     'version': '8'
                    },
                    'cpu': 'Apple A8X',
                    'ram': '8GB',
                    'colors': ['Gold', 'Silver', 'Space Gray'],
                    'models': [
                     {
                         'name': 'Wi-Fi',
                         'capacity': ['16GB', '64GB'],
                         'height': 240,
                         'width': 169.5,
                         'depth': 6.1,
                         'weight': 437,
                     },
                     {
                         'name': 'Wi-Fi + Cellular',
                         'capacity': ['16GB', '64GB'],
                         'height': 240,
                         'width': 169.5,
                         'depth': 6.1,
                         'weight': 444,
                     },
                    ],
                    'discontinued': False,
                    'videoUrl': None,
                    'non-ascii-attr': u'\u0442\u0435\u0441\u0442'
                },
                tags=['Tablets', 'Apple', 'Retina'])

    @classmethod
    def tearDownClass(cls):
        teardown_database(db)

    def test(self):
        with db_session:
            result = select(p for p in Product)[:]
            self.assertEqual(len(result), 1)
            p = result[0]
            p.info['os']['version'] = '9'
        with db_session:
            result = select(p for p in Product)[:]
            self.assertEqual(len(result), 1)
            p = result[0]
            self.assertEqual(p.info['os']['version'], '9')

    @db_session
    def test_query_int(self):
        val = get(p.info['display']['resolution'][0] for p in Product)
        self.assertEqual(val, 2048)

    @db_session
    def test_query_float(self):
        val = get(p.info['display']['size'] for p in Product)
        self.assertAlmostEqual(val, 9.7)

    @db_session
    def test_query_true(self):
        val = get(p.info['display']['multi-touch'] for p in Product)
        self.assertIs(val, True)

    @db_session
    def test_query_false(self):
        val = get(p.info['discontinued'] for p in Product)
        self.assertIs(val, False)

    @db_session
    def test_query_null(self):
        val = get(p.info['videoUrl'] for p in Product)
        self.assertIs(val, None)

    @db_session
    def test_query_list(self):
        val = get(p.info['colors'] for p in Product)
        self.assertListEqual(val, ['Gold', 'Silver', 'Space Gray'])
        self.assertNotIsInstance(val, TrackedValue)

    @db_session
    def test_query_dict(self):
        val = get(p.info['display'] for p in Product)
        self.assertDictEqual(val, {
            'size': 9.7,
            'resolution': [2048, 1536],
            'matrix-type': 'IPS',
            'multi-touch': True
        })
        self.assertNotIsInstance(val, TrackedValue)

    @db_session
    def test_query_json_field(self):
        val = get(p.info for p in Product)
        self.assertDictEqual(val['display'], {
            'size': 9.7,
            'resolution': [2048, 1536],
            'matrix-type': 'IPS',
            'multi-touch': True
        })
        self.assertNotIsInstance(val['display'], TrackedDict)
        val = get(p.tags for p in Product)
        self.assertListEqual(val, ['Tablets', 'Apple', 'Retina'])
        self.assertNotIsInstance(val, TrackedList)

    @db_session
    def test_get_object(self):
        p = get(p for p in Product)
        self.assertDictEqual(p.info['display'], {
            'size': 9.7,
            'resolution': [2048, 1536],
            'matrix-type': 'IPS',
            'multi-touch': True
        })
        self.assertEqual(p.info['discontinued'], False)
        self.assertEqual(p.info['videoUrl'], None)
        self.assertListEqual(p.tags, ['Tablets', 'Apple', 'Retina'])
        self.assertIsInstance(p.info, TrackedDict)
        self.assertIsInstance(p.info['display'], TrackedDict)
        self.assertIsInstance(p.info['colors'], TrackedList)
        self.assertIsInstance(p.tags, TrackedList)

    def test_set_str(self):
        with db_session:
            p = get(p for p in Product)
            p.info['os']['version'] = '9'
        with db_session:
            p = get(p for p in Product)
            self.assertEqual(p.info['os']['version'], '9')

    def test_set_int(self):
        with db_session:
            p = get(p for p in Product)
            p.info['display']['resolution'][0] += 1
        with db_session:
            p = get(p for p in Product)
            self.assertEqual(p.info['display']['resolution'][0], 2049)

    def test_set_true(self):
        with db_session:
            p = get(p for p in Product)
            p.info['discontinued'] = True
        with db_session:
            p = get(p for p in Product)
            self.assertIs(p.info['discontinued'], True)

    def test_set_false(self):
        with db_session:
            p = get(p for p in Product)
            p.info['display']['multi-touch'] = False
        with db_session:
            p = get(p for p in Product)
            self.assertIs(p.info['display']['multi-touch'], False)

    def test_set_null(self):
        with db_session:
            p = get(p for p in Product)
            p.info['display'] = None
        with db_session:
            p = get(p for p in Product)
            self.assertIs(p.info['display'], None)

    def test_set_list(self):
        with db_session:
            p = get(p for p in Product)
            p.info['colors'] = ['Pink', 'Black']
        with db_session:
            p = get(p for p in Product)
            self.assertListEqual(p.info['colors'], ['Pink', 'Black'])

    def test_list_del(self):
        with db_session:
            p = get(p for p in Product)
            del p.info['colors'][1]
        with db_session:
            p = get(p for p in Product)
            self.assertListEqual(p.info['colors'], ['Gold', 'Space Gray'])

    def test_list_append(self):
        with db_session:
            p = get(p for p in Product)
            p.info['colors'].append('White')
        with db_session:
            p = get(p for p in Product)
            self.assertListEqual(p.info['colors'], ['Gold', 'Silver', 'Space Gray', 'White'])

    def test_list_set_slice(self):
        with db_session:
            p = get(p for p in Product)
            p.info['colors'][1:] = ['White']
        with db_session:
            p = get(p for p in Product)
            self.assertListEqual(p.info['colors'], ['Gold', 'White'])

    def test_list_set_item(self):
        with db_session:
            p = get(p for p in Product)
            p.info['colors'][1] = 'White'
        with db_session:
            p = get(p for p in Product)
            self.assertListEqual(p.info['colors'], ['Gold', 'White', 'Space Gray'])

    def test_set_dict(self):
        with db_session:
            p = get(p for p in Product)
            p.info['display']['resolution'] = {'width': 2048, 'height': 1536}
        with db_session:
            p = get(p for p in Product)
            self.assertDictEqual(p.info['display']['resolution'], {'width': 2048, 'height': 1536})

    def test_dict_del(self):
        with db_session:
            p = get(p for p in Product)
            del p.info['os']['version']
        with db_session:
            p = get(p for p in Product)
            self.assertDictEqual(p.info['os'], {'type': 'iOS'})

    def test_dict_pop(self):
        with db_session:
            p = get(p for p in Product)
            p.info['os'].pop('version')
        with db_session:
            p = get(p for p in Product)
            self.assertDictEqual(p.info['os'], {'type': 'iOS'})

    def test_dict_update(self):
        with db_session:
            p = get(p for p in Product)
            p.info['os'].update(version='9')
        with db_session:
            p = get(p for p in Product)
            self.assertDictEqual(p.info['os'], {'type': 'iOS', 'version': '9'})

    def test_dict_set_item(self):
        with db_session:
            p = get(p for p in Product)
            p.info['os']['version'] = '9'
        with db_session:
            p = get(p for p in Product)
            self.assertDictEqual(p.info['os'], {'type': 'iOS', 'version': '9'})

    @db_session
    def test_set_same_value(self):
        p = get(p for p in Product)
        p.info = p.info

    @db_session
    def test_len(self):
        with raises_if(self, db.provider.dialect == 'Oracle',
                       TranslationError, 'Oracle does not provide `length` function for JSON arrays'):
            val = select(len(p.tags) for p in Product).first()
            self.assertEqual(val, 3)
            val = select(len(p.info['colors']) for p in Product).first()
            self.assertEqual(val, 3)

    @db_session
    def test_equal_str(self):
        p = get(p for p in Product if p.info['name'] == 'Apple iPad Air 2')
        self.assertTrue(p)

    @db_session
    def test_unicode_key(self):
        p = get(p for p in Product if p.info[u'name'] == 'Apple iPad Air 2')
        self.assertTrue(p)

    @db_session
    def test_equal_string_attr(self):
        p = get(p for p in Product if p.info['name'] == p.name)
        self.assertTrue(p)

    @db_session
    def test_equal_param(self):
        x = 'Apple iPad Air 2'
        p = get(p for p in Product if p.name == x)
        self.assertTrue(p)

    @db_session
    def test_composite_param(self):
        with raises_if(self, db.provider.dialect == 'Oracle',
                       TranslationError, "Oracle doesn't allow parameters in JSON paths"):
            key = 'models'
            index = 0
            val = get(p.info[key][index]['name'] for p in Product)
            self.assertEqual(val, 'Wi-Fi')

    @db_session
    def test_composite_param_in_condition(self):
        with raises_if(self, db.provider.dialect == 'Oracle',
                       TranslationError, "Oracle doesn't allow parameters in JSON paths"):
            key = 'models'
            index = 0
            p = get(p for p in Product if p.info[key][index]['name'] == 'Wi-Fi')
            self.assertIsNotNone(p)

    @db_session
    def test_equal_json_1(self):
        with raises_if(self, db.provider.dialect == 'Oracle', TranslationError,
                       "Oracle does not support comparison of json structures: "
                       "p.info['os'] == {'type':'iOS', 'version':'8'}"):
            p = get(p for p in Product if p.info['os'] == {'type': 'iOS', 'version': '8'})
            self.assertTrue(p)

    @db_session
    def test_equal_json_2(self):
        with raises_if(self, db.provider.dialect == 'Oracle', TranslationError,
                       "Oracle does not support comparison of json structures: "
                       "p.info['os'] == Json({'type':'iOS', 'version':'8'})"):
            p = get(p for p in Product if p.info['os'] == Json({'type': 'iOS', 'version': '8'}))
            self.assertTrue(p)

    @db_session
    def test_ne_json_1(self):
        with raises_if(self, db.provider.dialect == 'Oracle', TranslationError,
                       "Oracle does not support comparison of json structures: p.info['os'] != {}"):
            p = get(p for p in Product if p.info['os'] != {})
            self.assertTrue(p)
            p = get(p for p in Product if p.info['os'] != {'type': 'iOS', 'version': '8'})
            self.assertFalse(p)

    @db_session
    def test_ne_json_2(self):
        with raises_if(self, db.provider.dialect == 'Oracle', TranslationError,
                       "Oracle does not support comparison of json structures: p.info['os'] != Json({})"):
            p = get(p for p in Product if p.info['os'] != Json({}))
            self.assertTrue(p)
            p = get(p for p in Product if p.info['os'] != {'type': 'iOS', 'version': '8'})
            self.assertFalse(p)

    @db_session
    def test_equal_list_1(self):
        colors = ['Gold', 'Silver', 'Space Gray']
        with raises_if(self, db.provider.dialect == 'Oracle', TranslationError,
                       "Oracle does not support comparison of json structures: p.info['colors'] == Json(colors)"):
            p = get(p for p in Product if p.info['colors'] == Json(colors))
            self.assertTrue(p)

    @db_session
    @raises_exception(TypeError, "Incomparable types 'Json' and 'list' in expression: p.info['colors'] == ['Gold']")
    def test_equal_list_2(self):
        p = get(p for p in Product if p.info['colors'] == ['Gold'])

    @db_session
    def test_equal_list_3(self):
        with raises_if(self, db.provider.dialect == 'Oracle', TranslationError,
                       "Oracle does not support comparison of json structures: p.info['colors'] != Json(['Gold'])"):
            p = get(p for p in Product if p.info['colors'] != Json(['Gold']))
            self.assertIsNotNone(p)

    @db_session
    def test_equal_list_4(self):
        colors = ['Gold', 'Silver', 'Space Gray']
        with raises_if(self, db.provider.dialect == 'Oracle', TranslationError,
                       "Oracle does not support comparison of json structures: p.info['colors'] == Json(colors)"):
            p = get(p for p in Product if p.info['colors'] == Json(colors))
            self.assertTrue(p)

    @db_session
    @raises_exception(TypeError, "Incomparable types 'Json' and 'list' in expression: p.info['colors'] == []")
    def test_equal_empty_list_1(self):
        p = get(p for p in Product if p.info['colors'] == [])

    @db_session
    def test_equal_empty_list_2(self):
        with raises_if(self, db.provider.dialect == 'Oracle', TranslationError,
                       "Oracle does not support comparison of json structures: p.info['colors'] == Json([])"):
            p = get(p for p in Product if p.info['colors'] == Json([]))
            self.assertIsNone(p)

    @db_session
    def test_ne_list(self):
        with raises_if(self, db.provider.dialect == 'Oracle', TranslationError,
                       "Oracle does not support comparison of json structures: p.info['colors'] != Json(['Gold'])"):
            p = get(p for p in Product if p.info['colors'] != Json(['Gold']))
            self.assertTrue(p)

    @db_session
    def test_ne_empty_list(self):
        with raises_if(self, db.provider.dialect == 'Oracle', TranslationError,
                       "Oracle does not support comparison of json structures: p.info['colors'] != Json([])"):
            p = get(p for p in Product if p.info['colors'] != Json([]))
            self.assertTrue(p)

    @db_session
    def test_dbval2val(self):
        p = select(p for p in Product)[:][0]
        attr = Product.info
        val = p._vals_[attr]
        dbval = p._dbvals_[attr]
        self.assertIsInstance(dbval, str)
        self.assertIsInstance(val, TrackedValue)
        p.info['os']['version'] = '9'
        self.assertIs(val, p._vals_[attr])
        self.assertIs(dbval, p._dbvals_[attr])
        p.flush()
        self.assertIs(val, p._vals_[attr])
        self.assertNotEqual(dbval, p._dbvals_[attr])

    @db_session
    def test_wildcard_path_1(self):
        with raises_if(self, db.provider.dialect not in ('Oracle', 'MySQL'),
                       TranslationError, '...does not support wildcards in JSON path...'):
            names = get(p.info['models'][:]['name'] for p in Product)
            self.assertSetEqual(set(names), {'Wi-Fi', 'Wi-Fi + Cellular'})

    @db_session
    def test_wildcard_path_2(self):
        with raises_if(self, db.provider.dialect not in ('Oracle', 'MySQL'),
                       TranslationError, '...does not support wildcards in JSON path...'):
            values = get(p.info['os'][...] for p in Product)
            self.assertSetEqual(set(values), {'iOS', '8'})

    @db_session
    def test_wildcard_path_3(self):
        with raises_if(self, db.provider.dialect not in ('Oracle', 'MySQL'),
                       TranslationError, '...does not support wildcards in JSON path...'):
            names = get(p.info[...][0]['name'] for p in Product)
            self.assertSetEqual(set(names), {'Wi-Fi'})

    @db_session
    def test_wildcard_path_4(self):
        if db.provider.dialect == 'Oracle':
            raise unittest.SkipTest
        with raises_if(self, db.provider.dialect != 'MySQL',
                       TranslationError, '...does not support wildcards in JSON path...'):
            values = get(p.info[...][:][...][:] for p in Product)[:]
            self.assertSetEqual(set(values), {'16GB', '64GB'})

    @db_session
    def test_wildcard_path_with_params(self):
        if db.provider.dialect != 'Oracle':
            exc_msg = '...does not support wildcards in JSON path...'
        else:
            exc_msg = "Oracle doesn't allow parameters in JSON paths"
        with raises_if(self, db.provider.dialect != 'MySQL', TranslationError, exc_msg):
            key = 'models'
            index = 0
            values = get(p.info[key][:]['capacity'][index] for p in Product)
            self.assertListEqual(values, ['16GB', '16GB'])

    @db_session
    def test_wildcard_path_with_params_as_string(self):
        if db.provider.dialect != 'Oracle':
            exc_msg = '...does not support wildcards in JSON path...'
        else:
            exc_msg = "Oracle doesn't allow parameters in JSON paths"
        with raises_if(self, db.provider.dialect != 'MySQL', TranslationError, exc_msg):
            key = 'models'
            index = 0
            values = get("p.info[key][:]['capacity'][index] for p in Product")
            self.assertListEqual(values, ['16GB', '16GB'])

    @db_session
    def test_wildcard_path_in_condition(self):
        errors = {
            'MySQL': 'Wildcards are not allowed in json_contains()',
            'SQLite': '...does not support wildcards in JSON path...',
            'PostgreSQL': '...does not support wildcards in JSON path...'
        }
        dialect = db.provider.dialect
        with raises_if(self, dialect in errors, TranslationError, errors.get(dialect)):
            p = get(p for p in Product if '16GB' in p.info['models'][:]['capacity'])
            self.assertTrue(p)

    ##### 'key' in json

    @db_session
    def test_in_dict(self):
        obj = get(p for p in Product if 'resolution' in p.info['display'])
        self.assertTrue(obj)

    @db_session
    def test_not_in_dict(self):
        obj = get(p for p in Product if 'resolution' not in p.info['display'])
        self.assertIs(obj, None)
        obj = get(p for p in Product if 'xyz' not in p.info['display'])
        self.assertTrue(obj)

    @db_session
    def test_in_list(self):
        obj = get(p for p in Product if 'Gold' in p.info['colors'])
        self.assertTrue(obj)

    @db_session
    def test_not_in_list(self):
        obj = get(p for p in Product if 'White' not in p.info['colors'])
        self.assertTrue(obj)
        obj = get(p for p in Product if 'Gold' not in p.info['colors'])
        self.assertIs(obj, None)

    @db_session
    def test_var_in_json(self):
        with raises_if(self, db.provider.dialect == 'Oracle',
                       TypeError, "For `key in JSON` operation Oracle supports literal key values only, "
                                  "parameters are not allowed: key in p.info['colors']"):
            key = 'Gold'
            obj = get(p for p in Product if key in p.info['colors'])
            self.assertTrue(obj)

    @db_session
    def test_select_first(self):
        # query should not contain ORDER BY
        obj = select(p.info for p in Product).first()
        self.assertNotIn('order by', db.last_sql.lower())

    def test_sql_inject(self):
        # test quote in json is not causing error
        with db_session:
            p = select(p for p in Product).first()
            p.info['display']['size'] = "0' 9.7\""
        with db_session:
            p = select(p for p in Product).first()
            self.assertEqual(p.info['display']['size'], "0' 9.7\"")

    @db_session
    def test_int_compare(self):
        p = get(p for p in Product if p.info['display']['resolution'][0] == 2048)
        self.assertTrue(p)
        p = get(p for p in Product if p.info['display']['resolution'][0] != 2048)
        self.assertIsNone(p)
        p = get(p for p in Product if p.info['display']['resolution'][0] < 2048)
        self.assertIs(p, None)
        p = get(p for p in Product if p.info['display']['resolution'][0] <= 2048)
        self.assertTrue(p)
        p = get(p for p in Product if p.info['display']['resolution'][0] > 2048)
        self.assertIs(p, None)
        p = get(p for p in Product if p.info['display']['resolution'][0] >= 2048)
        self.assertTrue(p)

    @db_session
    def test_float_compare(self):
        p = get(p for p in Product if p.info['display']['size'] > 9.5)
        self.assertTrue(p)
        p = get(p for p in Product if p.info['display']['size'] < 9.8)
        self.assertTrue(p)
        p = get(p for p in Product if p.info['display']['size'] < 9.5)
        self.assertIsNone(p)
        p = get(p for p in Product if p.info['display']['size'] > 9.8)
        self.assertIsNone(p)

    @db_session
    def test_str_compare(self):
        p = get(p for p in Product if p.info['ram'] == '8GB')
        self.assertTrue(p)
        p = get(p for p in Product if p.info['ram'] != '8GB')
        self.assertIsNone(p)
        p = get(p for p in Product if p.info['ram'] < '9GB')
        self.assertTrue(p)
        p = get(p for p in Product if p.info['ram'] > '7GB')
        self.assertTrue(p)
        p = get(p for p in Product if p.info['ram'] > '9GB')
        self.assertIsNone(p)
        p = get(p for p in Product if p.info['ram'] < '7GB')
        self.assertIsNone(p)

    @db_session
    def test_bool_compare(self):
        p = get(p for p in Product if p.info['display']['multi-touch'] == True)
        self.assertTrue(p)
        p = get(p for p in Product if p.info['display']['multi-touch'] is True)
        self.assertTrue(p)
        p = get(p for p in Product if p.info['display']['multi-touch'] == False)
        self.assertIsNone(p)
        p = get(p for p in Product if p.info['display']['multi-touch'] is False)
        self.assertIsNone(p)
        p = get(p for p in Product if p.info['discontinued'] == False)
        self.assertTrue(p)
        p = get(p for p in Product if p.info['discontinued'] == True)
        self.assertIsNone(p)

    @db_session
    def test_none_compare(self):
        p = get(p for p in Product if p.info['videoUrl'] is None)
        self.assertTrue(p)
        p = get(p for p in Product if p.info['videoUrl'] is not None)
        self.assertIsNone(p)

    @db_session
    def test_none_for_nonexistent_path(self):
        p = get(p for p in Product if p.info['some_attr'] is None)
        self.assertTrue(p)

    @db_session
    def test_str_cast(self):
        p = get(coalesce(str(p.name), 'empty') for p in Product)
        last_sql = db.last_sql
        if db.provider.dialect == 'PostgreSQL':
            self.assertTrue(')::text' in last_sql)
        else:
            self.assertTrue('AS text' in db.last_sql)

    @db_session
    def test_int_cast(self):
        p = get(coalesce(int(p.info['os']['version']), 0) for p in Product)
        last_sql = db.last_sql
        if db.provider.dialect == 'PostgreSQL':
            self.assertTrue(')::int' in last_sql)
        else:
            self.assertTrue('as integer' in last_sql)


    def test_nonzero(self):
        with db_session:
            delete(p for p in Product)
            Product(name='P1', info=dict(id=1, val=True))
            Product(name='P2', info=dict(id=2, val=False))
            Product(name='P3', info=dict(id=3, val=0))
            Product(name='P4', info=dict(id=4, val=1))
            Product(name='P5', info=dict(id=5, val=''))
            Product(name='P6', info=dict(id=6, val='x'))
            Product(name='P7', info=dict(id=7, val=[]))
            Product(name='P8', info=dict(id=8, val=[1, 2, 3]))
            Product(name='P9', info=dict(id=9, val={}))
            Product(name='P10', info=dict(id=10, val={'a': 'b'}))
            Product(name='P11', info=dict(id=11))
            Product(name='P12', info=dict(id=12, val='True'))
            Product(name='P13', info=dict(id=13, val='False'))
            Product(name='P14', info=dict(id=14, val='0'))
            Product(name='P15', info=dict(id=15, val='1'))
            Product(name='P16', info=dict(id=16, val='""'))
            Product(name='P17', info=dict(id=17, val='[]'))
            Product(name='P18', info=dict(id=18, val='{}'))

        with db_session:
            val = select(p.info['id'] for p in Product if not p.info['val'])
            self.assertEqual(tuple(sorted(val)), (2, 3, 5, 7, 9, 11))

    @db_session
    def test_optimistic_check(self):
        p1 = Product.select().first()
        p1.info['foo'] = 'bar'
        flush()
        p1.name = 'name2'
        flush()
        p1.name = 'name3'
        flush()

    @db_session
    def test_avg(self):
        result = select(avg(p.info['display']['size']) for p in Product).first()
        self.assertAlmostEqual(result, 9.7)

    def test_pickle(self):
        with db_session:
            p1 = Product.select().first()
            data = pickle.dumps(p1)
        with db_session:
            p1 = pickle.loads(data)
            p1.name = 'name2'
            flush()
            rollback()



import unittest, warnings

from foalorm.orm import *
from foalorm.orm import core
from foalorm.orm.tests.testutils import raises_exception
from foalorm.orm.tests import db_params, teardown_database

db = Database()

class Person(db.Entity):
    id = PrimaryKey(int)
    name = Required(str)
    tel = Optional(str)


table_name = 'person'

class TestValidate(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        db.bind(**db_params)
        db.generate_mapping(check_tables=False)
        db.drop_all_tables(with_all_data=True)
        with db_session(ddl=True):
            db.execute("""
                create table "%s"(
                    id int primary key,
                    name text,
                    tel text
                )
            """ % table_name)

    @classmethod
    def tearDownClass(cls):
        teardown_database(db)

    @db_session
    def setUp(self):
        db.execute('delete from "%s"' % table_name)
        registry = getattr(core, '__warningregistry__', {})
        for key in list(registry):
            if type(key) is not tuple: continue
            text, category, lineno = key
            if category is DatabaseContainsIncorrectEmptyValue:
                del registry[key]

    @db_session
    def test_1a(self):
        with warnings.catch_warnings():
            warnings.simplefilter('ignore', DatabaseContainsIncorrectEmptyValue)
            db.insert(table_name, id=1, name='', tel='111')
            p = Person.get(id=1)
            self.assertEqual(p.name, '')

    @raises_exception(DatabaseContainsIncorrectEmptyValue,
                      'Database contains empty string for required attribute Person.name')
    @db_session
    def test_1b(self):
        with warnings.catch_warnings():
            warnings.simplefilter('error', DatabaseContainsIncorrectEmptyValue)
            db.insert(table_name, id=1, name='', tel='111')
            p = Person.get(id=1)

    @db_session
    def test_2a(self):
        with warnings.catch_warnings():
            warnings.simplefilter('ignore', DatabaseContainsIncorrectEmptyValue)
            db.insert(table_name, id=1, name=None, tel='111')
            p = Person.get(id=1)
            self.assertEqual(p.name, None)

    @raises_exception(DatabaseContainsIncorrectEmptyValue,
                      'Database contains NULL for required attribute Person.name')
    @db_session
    def test_2b(self):
        with warnings.catch_warnings():
            warnings.simplefilter('error', DatabaseContainsIncorrectEmptyValue)
            db.insert(table_name, id=1, name=None, tel='111')
            p = Person.get(id=1)


if __name__ == '__main__':
    unittest.main()



import unittest

from foalorm.orm import *
from foalorm.orm.tests import setup_database, teardown_database, only_for

db = Database()

class X(db.Entity):
    id = PrimaryKey(int)
    parent = Optional('X', reverse='children')
    children = Set('X', reverse='parent', cascade_delete=True)


class Y(db.Entity):
    parent = Optional('Y', reverse='children')
    children = Set('Y', reverse='parent', cascade_delete=True, lazy=True)


class TestCascade(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        setup_database(db)
        with db_session:
            x1 = X(id=1)
            x2 = X(id=2, parent=x1)
            x3 = X(id=3, parent=x1)
            x4 = X(id=4, parent=x3)
            x5 = X(id=5, parent=x3)
            x6 = X(id=6, parent=x5)
            x7 = X(id=7, parent=x3)
            x8 = X(id=8, parent=x7)
            x9 = X(id=9, parent=x7)
            x10 = X(id=10)
            x11 = X(id=11, parent=x10)
            x12 = X(id=12, parent=x10)

            y1 = Y(id=1)
            y2 = Y(id=2, parent=y1)
            y3 = Y(id=3, parent=y1)
            y4 = Y(id=4, parent=y3)
            y5 = Y(id=5, parent=y3)
            y6 = Y(id=6, parent=y5)
            y7 = Y(id=7, parent=y3)
            y8 = Y(id=8, parent=y7)
            y9 = Y(id=9, parent=y7)
            y10 = Y(id=10)
            y11 = Y(id=11, parent=y10)
            y12 = Y(id=12, parent=y10)

    @classmethod
    def tearDownClass(cls):
        teardown_database(db)

    def setUp(self):
        rollback()
        db_session.__enter__()

    def tearDown(self):
        rollback()
        db_session.__exit__()

    def test_1(self):
        db.merge_local_stats()
        X[1].delete()
        stats = db.local_stats[None]
        self.assertEqual(5, stats.db_count)

    def test_2(self):
        db.merge_local_stats()
        Y[1].delete()
        stats = db.local_stats[None]
        self.assertEqual(10, stats.db_count)
